from builtins import issubclass
import sys
from types import MethodType

from pyson.Deserializer import Deserializer
from pyson.JsonTools import str_to_class
from typing import Optional


def JsonDeserialize(using:Optional[type]):
    '''
    Annotation telling pyson to use a custom deserializer.
    
    @param using the class to be used for the property in json.
    '''
    if not (using==None or issubclass(using, Deserializer)):
        raise ValueError("@JsonDeserializer takes string with class.path of deserializer but got "+str( using)+" of type "+str(type(using)))
    
    def doAnnotate(annotatedfunction):
        '''
        This function is applied to the annotated class.
        It just adds an attribute __jsondeserialize__ to the class
        with using as value
        
        This works by setting an attr to the annotated function.
        The value may be none (which means use the default )

        @param annotatedfunction the function that gets this annotation
        '''
        setattr(annotatedfunction, '__jsondeserialize__', using)
        return annotatedfunction
    
    # we return doAnnotate as result of the function.
    # that way we can apply that function to the next argument
    # which is the annotatedclass
    return doAnnotate


def getDeserializer(ob:object)->object : # Class
    '''
    @param ob the class or instance presumably containing deserializer annotation.
    @returns the 'using'  clas  to the deserializing class, 
    or None if ob has no deserialize annotation.
    @raise  ValueError if the 'using' class is not a Deserializer
    '''
    if not hasattr(ob, '__jsondeserialize__'):
        return None
    
    return getattr(ob, '__jsondeserialize__')
