from builtins import issubclass
import sys
from types import MethodType

from pyson.Serializer import Serializer
from pyson.JsonTools import str_to_class
from typing import Optional, cast


def JsonSerialize(using:Optional[type]):
    '''
    Gives an explicit serializer to be used
    
    @param using the serializer to use.
    '''
    if not (using==None or issubclass(using, Serializer)):
        raise ValueError("@JsonSerializer requires class of type Serializer but got "+str( using)+" of type "+str(type(using)))

    def doAnnotate(annotatedfunction):
        '''
        This function is applied to the annotated class.
        It just adds an attribute __jsonserialize__ to the class
        with using as value
        
        This works by setting an attr to the annotated class/function.
        The value may be none (which means use the default )
        
        Note that the if not set, the annotation value is inherited from parent

        @param annotatedfunction the function that gets this annotation
        '''
        setattr(annotatedfunction, '__jsonserialize__', using)
        return annotatedfunction
    
    # we return doAnnotate as result of the function.
    # that way we can apply that function to the next argument
    # which is the annotatedclass
    return doAnnotate


def getSerializer(ob:object)->Serializer : 
    '''
    @param ob the class or instance possibly containing serializer annotation.
    @returns the 'using'  clas  to the deserializing class, 
    or None if ob has no deserialize annotation.
    @raise  ValueError if the 'using' class is not a Deserializer
    '''
    if not hasattr(ob, '__jsonserialize__'):
        return None
    
    return getattr(ob, '__jsonserialize__')
   